@@ -20,18 +20,18 @@ package
 package PkgConfig;
 
 #First two digits are Perl version, second two are pkg-config version
-our $VERSION = '0.08220';
-
-require 5.006;
+our $VERSION = '0.08420';
 
 use strict;
 use warnings;
+use 5.006;
 use Config;
 use File::Spec;
 use File::Glob 'bsd_glob';
 use Class::Struct; #in core since 5.004
 use Data::Dumper;
 use File::Basename qw( dirname );
+use Text::ParseWords qw( shellwords );
 
 our $UseDebugging;
 
@@ -376,6 +376,10 @@ struct(
      'print_variable' => '$',
      'print_values' => '$',
      'defined_variables' => '*%',
+     
+     # for creating PkgConfig objects with identical
+     # settings
+     'original' => '$',
     ]
 );
 
@@ -387,12 +391,12 @@ struct(
 
 sub _get_pc_varname {
     my ($self,$vname_base) = @_;
-    return $self->varclass . "::" . $vname_base;
+    $self->varclass . "::" . $vname_base;
 }
 
 sub _get_pc_udefname {
     my ($self,$vname_base) = @_;
-    return $self->udefclass . "::" . $vname_base;
+    $self->udefclass . "::" . $vname_base;
 }
 
 sub _pc_var {
@@ -401,9 +405,11 @@ sub _pc_var {
     no strict 'refs';
     $vname = $self->_get_pc_varname($vname);
     my $glob = *{$vname};
-    return unless $glob;
-    
-    return $$glob;
+    $glob ? $$glob : ();
+}
+
+sub _quote_cvt($)  {
+    join ' ', map { s/(\s|"|')/\\$1/g; $_ } shellwords(shift)
 }
 
 sub assign_var {
@@ -415,7 +421,7 @@ sub assign_var {
         log_debug("Prefix already defined by user");
         return;
     }
-    my $evalstr = sprintf('$%s = %s',
+    my $evalstr = sprintf('$%s = PkgConfig::_quote_cvt(%s)',
                     $self->_get_pc_varname($field), $value);
     
     log_debug("EVAL", $evalstr);
@@ -455,6 +461,8 @@ sub find {
         ['exclude_cflags', \@DEFAULT_EXCLUDE_CFLAGS]
     );
     
+    my %original = %options;
+    
     foreach (@uspecs) {
         my ($basekey,$default) = @$_;
         my $list = [ @{$options{$basekey} ||= [] } ];
@@ -471,6 +479,7 @@ sub find {
     $VarClassSerial++;
     $options{varclass} = sprintf("PkgConfig::Vars::SERIAL_%d", $VarClassSerial);
     $options{udefclass} = sprintf("PkgConfig::UDefs::SERIAL_%d", $VarClassSerial);
+    $options{original} = \%original;
     
     
     my $udefs = delete $options{VARS} || {};
@@ -520,7 +529,7 @@ sub find {
         }
     }
     
-    return $o;
+    $o;
 }
 
 ################################################################################
@@ -601,14 +610,14 @@ sub get_requires {
         }
     }
     #log_debug(@ret);
-    return @ret;
+    @ret;
 }
 
 
 sub parse_line {
     my ($self,$line,$evals) = @_;
     no strict 'vars';
-    
+
     $line =~ s/#[^#]+$//g; # strip comments
     return unless $line;
     
@@ -649,14 +658,17 @@ sub parse_line {
     
     $value =~ s/\$\{/\$\{$varclass\::/g;
     
+    # preserve quoted space
+    $value = join ' ', map { s/(["'])/\\$1/g; "'$_'" } shellwords $value
+      if $value =~ /[\\"']/;
+    
     #quoute the value string, unless quouted already
-    $value = "\"$value\"" unless $value =~ /^["']/;
+    $value = "\"$value\"";
     
     #get existent variables from our hash:
     
     
-    $value =~ s/'/"/g; #allow for interpolation
-    
+    #$value =~ s/'/"/g; #allow for interpolation
     $self->assign_var($field, $value);
     
 }
@@ -712,11 +724,18 @@ sub parse_pcfile {
         $self->pkg_description( $self->_pc_var('description') );
         $self->pkg_exists(1);        
     }
-    
+
     unless ($self->no_recurse) {
         foreach (@deps) {
             my ($dep,$cmp_op,$version) = @$_;
-            $self->find_pcfile($dep);
+            $dep = "$dep $cmp_op $version" if defined $cmp_op;
+            my $other = PkgConfig->find($dep, %{ $self->original });
+            if($other->errmsg) {
+                $self->errmsg($other->errmsg);
+                last;
+            }
+            $self->append_cflags(  $other->get_cflags );
+            $self->append_ldflags( $other->get_ldflags );
         }
     }
 }
@@ -764,13 +783,17 @@ sub find_pcfile {
 ################################################################################
 ################################################################################
 
+sub _return_context (@) {
+    wantarray ? (@_) : join(' ', map { s/(\s|['"])/\\$1/g; $_ } @_)
+}
+
 sub get_cflags {
     my $self = shift;
     my @cflags = @{$self->cflags};
 
     filter_omit(\@cflags, $self->exclude_cflags);
     filter_dups(\@cflags);
-    return @cflags;
+    _return_context @cflags;
 }
 
 sub get_ldflags {
@@ -790,7 +813,7 @@ sub get_ldflags {
     @ret = reverse @ret;
     filter_dups(\@ret);
     @ret = reverse(@ret);
-    return @ret;
+    _return_context @ret;
 }
 
 sub get_var {
@@ -812,7 +835,7 @@ sub get_list {
             }
         }
     }
-    return @rv;
+    @rv;
 }
 
 
@@ -831,10 +854,11 @@ sub _split_flags {
     if(@flags == 1) {
         my $str = shift @flags;
         return () if !$str;
-        @flags = split(/\s+/, $str);
+        #@flags = map { s/\\(\s)/$1/g; $_ } split(/(?<!\\)\s+/, $str);
+        @flags = shellwords $str;
     }
     @flags = grep $_, @flags;
-    return @flags;
+    @flags;
 }
 
 
@@ -879,7 +903,7 @@ sub version_2_array {
         && $chunk =~ /^\d+$/) {
         push @ret, $chunk;
     }
-    return @ret;
+    @ret;
 }
 
 
@@ -897,7 +921,7 @@ sub version_check {
             return 0;
         }
     }
-    return 1;
+    1;
 }
 
 
@@ -1136,7 +1160,7 @@ if($PrintLibsOnlyl or ($PrintLibsOnlyl and $PrintLibsOnlyL)) {
     @print_flags = grep /^-[LR]/, $o->get_ldflags;
 }
 
-print $_, " " for @print_flags;
+print scalar PkgConfig::_return_context(@print_flags);
 print "\n";
 exit(0);
 
@@ -1390,7 +1414,15 @@ The version of the package
 
 =head4 I<< $o->get_ldflags >>
 
-Returns a list of compiler and linker flags, respectively.
+Returns compiler and linker flags, respectively.
+
+In list context, these methods return a list with each argument split on
+unescaped spaces.
+
+In list context returns a list of compiler and linker flags, respectively.
+
+In scalar context returns a string of compiler and linker flags with spaces
+and quotes escaped correctly.
 
 =head4 I<< $o->get_var($name) >>
 
@@ -1522,6 +1554,8 @@ Other contributors include:
 
 =over 4
 
+=item Vladimir Timofeev (vovkasm)
+
 =item kmx
 
 =item Sanel Zukan
